@@ -1017,7 +1017,9 @@ test_IdnaTest(char *linep)
 {
   char *type, *source, *toUnicode, *toASCII, *NV8, *org_source;
   int expected_toASCII_failure;
+#if HAVE_LIBUNISTRING
   extern int _libunistring_version;
+#endif
 
   type = _nextField(&linep);
   org_source = source = _nextField(&linep);
@@ -1039,14 +1041,17 @@ test_IdnaTest(char *linep)
 
   printf("##########%s#%s#%s#%s#%s#\n", type, org_source, toUnicode, toASCII, NV8);
 
+#if HAVE_LIBUNISTRING
   /* 3 tests fail with libunicode <= 0.9.3 - just skip them until we have a newer version installed */
   /* watch out, libunicode changed versioning scheme up from 0.9.4 */
+  /* If !HAVE_LIBUNISTRING, we use internal gnulib code which works. */
   if (_libunistring_version <= 9) {
     if (!strcmp(toASCII, "xn--8jb.xn--etb875g")) {
       free(source);
       return 0;
     }
   }
+#endif
 
   if (*type == 'B') {
     _check_toASCII(source, toASCII, 1, expected_toASCII_failure);
@@ -1088,7 +1093,7 @@ main (int argc, const char *argv[])
   separator();
 
   // test all IDNA cases from Unicode 9.0.0
-  if (_scan_file(argc == 1 ? "IdnaTest.txt" : argv[1], test_IdnaTest))
+  if (_scan_file(argc == 1 ? SRCDIR"/IdnaTest.txt" : argv[1], test_IdnaTest))
     return EXIT_FAILURE;
 
   separator();
